'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.moveResources = moveResources;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @description transfer an entire resource from source to target directory
 * @param {string} resourceSourcePath - current position of resource
 * @param {string} resourceTargetPath - folder where resources are moved
 * @return {Promise} Move directory Promise
 */
function moveResources(resourceSourcePath, resourceTargetPath) {
  return new Promise((resolve, reject) => {
    if (resourceSourcePath && resourceSourcePath.length && resourceTargetPath && resourceTargetPath.length && _fsExtra2.default.pathExistsSync(resourceSourcePath)) {
      if (_fsExtra2.default.pathExistsSync(resourceTargetPath)) {
        _fsExtra2.default.removeSync(resourceTargetPath);
      }
      _fsExtra2.default.ensureDirSync(_path2.default.dirname(resourceTargetPath));
      _fsExtra2.default.copySync(resourceSourcePath, resourceTargetPath);
      _fsExtra2.default.removeSync(resourceSourcePath);
      resolve(resourceTargetPath);
    } else {
      reject('Invalid parameters to moveResources()');
    }
  });
}